function GetAngle(x1,y1,x2,y2)
	return math.deg(math.atan2((y1-y2),(x1-x2)));
end

function CosXMag(Mag,Ang)
	return Mag * math.cos(Ang*math.pi/180);
end

function SinYMag(Mag,Ang)
	return Mag * math.sin(Ang*math.pi/180);
end

function RandPlusMinus(Integer)
	local Rand = math.random(0,1);
	local Num = Integer;
	if Rand == 1 then
		Num = Num * (-1)
	else
		Num = Num * (1)
	end
	return Num;
end

function DrawLine(StartPosition, EndPosition, DrawLengthMultiplier, DrawingQuality, MOPixelToDrawWith)	
	local VectorToDestination = SceneMan:ShortestDistance(StartPosition,EndPosition,true) * DrawLengthMultiplier;
	local ParticleCount = VectorToDestination.Magnitude * DrawingQuality;
	local DistanceMultiplier = (1 / DrawingQuality);
	for ParticleNumber = 0, ParticleCount, 1 do
		local Particle = CreateMOPixel(MOPixelToDrawWith);
		Particle.Pos = StartPosition + VectorToDestination.Normalized * DistanceMultiplier * ParticleNumber;
		MovableMan:AddParticle(Particle);
	end
end

-- DrawFracturedLine - Draws a fractured (Zig Zag) line
-- Arguments:
-- StartPosition - A vector, the starting position for the line
-- EndPosition - A vector, the ending position for the line
-- AmountOfFractures - The amount of fractures the line will have
-- ExcessSegmentLength - The excess length for each segment, 1 is no excess pixels, 1.2 is excess pixels equal to one fifth (0.2) of the segment, 2 is excess pixels equal to twice the length of the segment, etc.
-- DrawingQuality - The quality of the line drawn
-- DeviationFromCenter - How far will the fractures deviate from the original line
-- MOPixelToDrawWith - The MOPixel to use to draw the line

function DrawFracturedLine(StartPosition, EndPosition, AmountOfFractures, ExcessSegmentLength, DrawingQuality, DeviationFromCenter, MOPixelToDrawWith) -- Draws a fractured line
	local VectorToDestination = SceneMan:ShortestDistance(StartPosition,EndPosition,true); -- Creates a vector starting at the start position and ending at the end position
	local DistBetweenFracts = VectorToDestination.Magnitude / AmountOfFractures; -- The distance between fractures
	local List = {}; -- Creates the list that will store our fracture points' positions 
	for ParticleNumber = 0, AmountOfFractures, 1 do
		if ParticleNumber == 0 then  -- If it is the first particle in our chain
			local Particle = CreateMOPixel(MOPixelToDrawWith);  -- Set it as 'MOPixelToDrawWith'
			Particle.Pos = StartPosition;  -- Set its position as the start position
			MovableMan:AddParticle(Particle);  -- Create it
			List[ParticleNumber] = Particle;  -- And add it to our list. What all this does is makes sure the first particle is always in the same place, the start position
		else  -- If it is any other particle in our chain
			local Rand = DeviationFromCenter - math.random(0,DeviationFromCenter) * RandPlusMinus(1);  -- Creates a random number between -DeviationFromCenter and DeviationFromCenter
			local Offset = VectorToDestination.Normalized.Perpendicular * Rand;  -- and multiplies a perpendicular vector to VectorToDestination to by it  
			local Particle = CreateMOPixel(MOPixelToDrawWith);  -- Set the particle as 'MOPixelToDrawWith'
			Particle.Pos = StartPosition + VectorToDestination.Normalized * DistBetweenFracts * ParticleNumber + Offset; -- Assign a possition to our particle, and add the random vector to it
			MovableMan:AddParticle(Particle);  -- Create it
			List[ParticleNumber] = Particle;  -- And add it to our list
		end
	end
	for i = 0, #List, 1 do
		local nexti = i + 1;  -- i is the current particle in our list, nexti is the particle after i
		if nexti <= #List then  -- If nexti exists
			DrawLine(List[i].Pos, List[nexti].Pos, ExcessSegmentLength, DrawingQuality, MOPixelToDrawWith);  -- Draw a line from i to nexti
		end
	end
end

function Create(self)

-- Gets the parent
	local CurDist = 40
	for p in MovableMan.Actors do
		local dist = SceneMan:ShortestDistance(self.Pos, p.Pos, true).Magnitude;
		if dist < CurDist then
			CurDist = dist;
			self.Parent = p;
		end
	end

-- Deducts 10 oz from the parent's funds
	local tmp = ActivityMan:GetActivity():GetTeamFunds(self.Parent.Team);
	ActivityMan:GetActivity():SetTeamFunds(tmp - 10, self.Parent.Team);

-- This will be set as our target, currently its nil
	self.Target = nil;
-- The range in pixels of our weapon
	self.Arcdist = 200;
-- If the weapon should chain or not
	self.Chain = nil;

-- Create a vector in the direction we're firing and the leangth of our range
	self.Range = Vector(0,0);
	self.Range.X = math.cos(self.RotAngle) * self.Arcdist;
	self.Range.Y = math.sin(self.RotAngle) * self.Arcdist * -1;

-- Check to see if we hit something, if we did set it as our target, and chain
	local j = SceneMan:CastMORay(self.Pos, self.Range, self.Parent.ID, 0, true, 5);
	if j ~= 255 then
		self.Target = MovableMan:GetMOFromID(MovableMan:GetMOFromID(j).RootID);
		self.Chain = true;
	end

-- If we didn't hit, draw a nice line in the direction we fired
	if self.Target == nil then
		local Hit = Vector(0,0);
		HitTerrain = SceneMan:CastStrengthRay(self.Pos, self.Range * 2, 0, Hit, 0, 0, true);
		if Hit ~= nil then
			DrawFracturedLine(self.Pos, Hit, 20, 1.25, 1, 2, "Laser Pixel");
		end
	end
end

function Update(self)
-- If we hit and our target is still alive
	if self.Chain == true and MovableMan:ValidMO(self.Target) then

-- We draw a line to the target
		DrawFracturedLine(self.Pos, self.Target.Pos, 20, 1.25, 1, 2, "Laser Pixel");

-- We move to the target's position
		self.Pos = self.Target.Pos;

-- We create an explosion at the target's position
		local Bang = CreateMOSRotating("Explosion");
		Bang.Pos = self.Target.Pos;
		MovableMan:AddParticle(Bang);
		Bang:GibThis();

-- We hit something! Add 5 oz to our parent's funds as a reward
		local tmp = ActivityMan:GetActivity():GetTeamFunds(self.Parent.Team);
		ActivityMan:GetActivity():SetTeamFunds(tmp + 5, self.Parent.Team);

-- Remember the previous target as "OldTarget"
		self.OldTarget = self.Target
		self.Target = nil;

-- Find a new target that isn't the old one and isn't the parent
		local CurDist = self.Arcdist
		for actor in MovableMan.Actors do
			local dist = SceneMan:ShortestDistance(self.Pos, actor.Pos, true).Magnitude;
			if dist < CurDist and actor.ID ~= self.Parent.ID and actor.ID ~= self.OldTarget.ID then
				CurDist = dist;
				self.Target = actor;
			end
		end

-- If we've found a target, continue the chain, otherwise stop it
		if self.Target ~= nil then
			self.Chain = true;
		else
			self.Chain = false;
		end
	else
		MovableMan:RemoveParticle(self);
	end
end